#include "Estacoes.h"
#include <stdio.h>


//#define WINDOWS_VS 1

#define _MAX(x,y) (x>y?x:y)

// dados da instncia (estticos)
int CEstacao::largura, CEstacao::altura; 
int CEstacao::populacao[MAX_ALTURA][MAX_LARGURA]; 

// configuraes
int CEstacao::distMin = 2; // valor terico, com base no custo 
int CEstacao::familias = 1; // nmero total de famlias (atualizar ao carregar a instncia)

CEstacao::CEstacao(void)
{
}

CEstacao::~CEstacao(void)
{
}

// Cria um objecto que e uma copia deste
TProcuraConstrutiva* CEstacao::Duplicar(void)
{
	CEstacao *clone=new CEstacao;
	clone->Copiar(this);
	return clone;
}

void CEstacao::Copiar(TProcuraConstrutiva* objecto) {
	TProcuraMelhorativa::Copiar(objecto);
	estacao = ((CEstacao*)objecto)->estacao;
}

// Coloca o objecto no estado inicial da procura
void CEstacao::SolucaoVazia(void) {
	estacao.Count(0);
	custo = -1;
}

// retorna a distncia at  estao mais prxima
int CEstacao::Distancia(TZona &zona)
{
	int distancia = 100;
	for (int i = 0; i < estacao.Count(); i++)
		distancia = _MIN(distancia, estacao[i].Distancia(zona));
	return distancia;
}

// retorna o custo com base na distncia at  estao mais prxima
int CEstacao::Custo(TZona &zona)
{
	int distancia = Distancia(zona);
	if (distancia < 2)
		return 0;
	if (distancia > 5)
		return 10;
	return 1 << (distancia - 2);
}


// Coloca em sucessores a lista de objectos sucessores (sao alocados neste metodo e tem de ser apagados)
// O custo nao necessita de ser preenchido, caso seja sempre unitario
void CEstacao::Sucessores(TVector<TProcuraConstrutiva*>&sucessores, TVector<int>&custo)
{
	TZona zona;
	// colocar todas as posies, aps a ltima posio (de modo e evitar repetio)
	// no utilizar posies a menos de distMin at outra estao

	for (zona.linha = (estacao.Count() == 0 ? 1 :
		estacao.Last().linha); zona.linha < altura - 1; zona.linha++)
		for (zona.coluna = (estacao.Count() == 0 ? 1 :
			estacao.Last().linha); zona.coluna < largura - 1; zona.coluna++)
			// verificar a distncia at  estao mais prxima
			if (Distancia(zona) > distMin) {
				CEstacao* sucessor = (CEstacao*)Duplicar();
				sucessor->estacao.Add(zona);
				sucessor->custo = -1;
				sucessores.Add(sucessor);
			}

	TProcuraConstrutiva::Sucessores(sucessores,custo);
}

// Escrever informacao de debug sobre o objecto currente
// (utilizar variavel TProcuraConstrutiva::debug para seleccionar o detalhe pretendido)
void CEstacao::Debug(void)
{
	for (int i = 0; i < altura; i++) {
		NovaLinha();
		for (int j = 0; j < largura; j++) {
			char c=' ';
			TZona z(i, j);
			if (estacao.Find(z) >= 0)
				c = '#';
			printf("%3d%c", populacao[i][j], c);
		}
	}
	NovaLinha();
	printf("Custo: %d", Avaliar());
}

// Retorna verdade caso o estado actual seja um estado objectivo
bool CEstacao::SolucaoCompleta(void)
{
	// verificar se a mdia da distncia  inferior a 4
	return estacao.Count() > 0 && Avaliar() % 1000 < 300;
}

int CEstacao::Heuristica(void)
{
	int resultado = Avaliar(); // retorna o custo atual
	if (!SolucaoCompleta()) {
		// adicionar custo caso a ltima estao no esteja perto de zonas com populao
		// subtrair as populaes a distncia 1
		if (estacao.Count() > 0) {
			TZona zona;
			for (zona.linha = estacao.Last().linha - 1;
				zona.linha <= estacao.Last().linha + 1; zona.linha++) 
				for (zona.coluna = estacao.Last().coluna- 1;
					zona.coluna<= estacao.Last().coluna + 1; zona.coluna++) 
					resultado -= populacao[zona.linha][zona.coluna];
		}
	}
	return resultado;
}

// Retorna o valor da solucao completa actual.
// custo: #estacoes * 1000 + Media(custo)*100
int CEstacao::Avaliar(void)
{
	int custoTotal = 0;
	TZona zona;
	if (custo > 0 && custo < 10000) // estado j avaliado
		return custo;
	if (estacao.Count() == 0)
		return custo = 9999;
	for (zona.linha = 0; zona.linha < altura; zona.linha++)
		for (zona.coluna = 0; zona.coluna < largura; zona.coluna++)
			if (populacao[zona.linha][zona.coluna] > 0)
				custoTotal += populacao[zona.linha][zona.coluna] * Custo(zona);

	TProcuraMelhorativa::Avaliar();	
	
	//return custo = estacao.Count() * 1000 + 100.0 * custoTotal / familias + .5; // houve quem arredonde para o mais prximo 
	return custo = estacao.Count() * 1000 + 100 * custoTotal / familias;
}

void CEstacao::NovaSolucao(void)
{
	SolucaoVazia();
	while (!SolucaoCompleta()) { 
		TZona nova;
		// escolher uma nova possio para uma estao aleatoriamente
		nova.linha = 1 + TRand::rand() % (altura-2);
		nova.coluna = 1 + TRand::rand() % (largura - 2);
		if (Distancia(nova) > distMin) {
			estacao.Add(nova);
			custo = -1; // forar avaliao
		}
	}
}

// mover uma estao uma unidade para qualquer direo
void CEstacao::Vizinhanca(TVector<TProcuraMelhorativa*>& vizinhos)
{
	for (int i = 0; i < estacao.Count(); i++) {
		TZona mover;
		// mover a estao
		mover = estacao[i];
		estacao.Delete(i);
		for (int di = -1; di < 2; di++)
			if (mover.linha + di >= 1 && mover.linha + di < altura - 1)
				for (int dj = -1; dj < 2; dj++)
					if (mover.coluna + dj >= 1 && mover.coluna + dj < largura - 1) 
					{
						if (di == 0 && dj == 0) {
							if (estacao.Count() > 0) { // considerar a possibilidade de remover a estao, se no for nica
								int backupCusto = custo;
								custo = -1;	// atualiza custo sem a estao i
								if (SolucaoCompleta()) { // vizinho OK, pode-se remover a estao
									vizinhos.Add((TProcuraMelhorativa*)Duplicar());
									vizinhos.Last()->custo = custo; // poupar uma avaliao, que acabou de ser realizada
								}
								custo = backupCusto;
							}
						}
						else {
							TZona novo(mover);
							novo.linha += di;
							novo.coluna += dj;
							if (Distancia(novo) > distMin) {
								CEstacao* vizinho = (CEstacao*)Duplicar();
								vizinho->estacao.Add(novo);
								vizinho->custo = -1;
								if (vizinho->SolucaoCompleta())
									vizinhos.Add(vizinho);
								else
									delete vizinho;
							}
						}
					}
		estacao.Insert(mover, i);
	}
	TProcuraMelhorativa::Vizinhanca(vizinhos);
}

void CEstacao::Mutar(void)
{
	// executar movimentos das estaes com 50%
	do {
		int i = TRand::rand() % estacao.Count();
		TZona mover;
		mover = estacao[i];
		estacao.Delete(i);
		int di = (TRand::rand() % 3) - 1;
		int dj = (TRand::rand() % 3) - 1;
		int backupCusto = custo;
		if (estacao.Count()>0 && di == 0 && dj == 0) {
			// remover a estao, caso seja possvel
			custo = -1; // atualiza custo sem a estao i
			if (!SolucaoCompleta()) { // remoo OK
				// manter a estao sem alteraes
				estacao.Insert(mover, i);
				custo = backupCusto; // repor o custo
			}
		}
		else {
			TZona novo(mover);
			novo.linha += di;
			novo.coluna += dj;
			if (Distancia(novo) > distMin) {
				mover.linha += di;
				mover.coluna += dj;
				estacao.Insert(mover, i);
				custo = -1;
				if (!SolucaoCompleta()) {
					// repor a posio original
					mover.linha -= di;
					mover.coluna -= dj;
					estacao[i] = mover;
					custo = backupCusto; // repor o custo
				}
			}
		}
	} while (TRand::rand() % 2);
}

void CEstacao::Cruzamento(TProcuraMelhorativa* a, TProcuraMelhorativa* b)
{
	TVector<TZona> todasEstacoes;
	estacao.Count(0);
	custo = -1;
	// colocar estaes de ambos os elementos
	todasEstacoes = ((CEstacao*)a)->estacao;
	todasEstacoes += ((CEstacao*)b)->estacao;
	todasEstacoes.RandomOrder(); // aproveitar as primeiras
	// parar assim que a soluo satisfaa as restries
	while (!SolucaoCompleta() && todasEstacoes.Count()>0) {
		estacao.Add(todasEstacoes.Pop());
		custo = -1;
	}

	TProcuraMelhorativa::Cruzamento(a, b);
}

int CEstacao::Distancia(TProcuraMelhorativa* a)
{
	int total = 10 * abs(estacao.Count() - ((CEstacao*)a)->estacao.Count());
	// somar a distncia de cada estao s estaes do outro estado (10 pela diferena entre nmero de estaes)
	for (int i = 0; i < estacao.Count(); i++)
		total += ((CEstacao*)a)->Distancia(estacao[i]);
	return total;
}


void CEstacao::CarregarInstancia(int id)
{
	static int populacao[20][MAX_ALTURA][MAX_LARGURA] = {
	// 5x5
{
{0,7,0,0,4},
{0,0,0,4,0},
{1,0,0,0,0},
{4,4,1,0,0},
{6,0,3,4,4},
},
{
{4,0,0,10,1},
{1,0,0,0,0},
{0,0,1,6,3},
{0,4,0,0,2},
{8,0,6,3,0},
},
		// 7x7
{
{0,8,0,4,5,10,0},
{0,4,0,7,0,4,0},
{0,2,4,2,0,0,2},
{0,7,0,1,2,0,0},
{2,4,0,0,3,0,2},
{0,4,0,0,3,0,0},
{2,0,0,0,0,0,0},
},
{
{0,0,1,0,7,0,1},
{0,1,4,0,0,0,4},
{0,0,0,0,2,0,0},
{3,1,0,8,5,7,7},
{0,4,0,3,0,0,0},
{0,0,0,3,2,4,2},
{0,8,3,6,3,0,0},
},
		// 9x9
{
{6,7,2,0,0,0,0,0,0},
{3,3,6,0,8,4,3,1,0},
{0,0,8,0,0,0,2,4,0},
{0,0,0,1,0,3,2,0,0},
{0,0,0,7,4,0,1,0,0},
{12,8,0,5,4,1,4,3,4},
{8,0,1,2,4,3,3,0,0},
{1,1,0,0,0,0,5,0,0},
{4,0,0,0,4,6,0,13,2},
},
{
{0,0,0,0,0,0,0,0,0},
{4,0,8,4,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0},
{0,0,0,0,3,0,0,1,0},
{0,3,0,0,0,0,0,0,0},
{0,0,0,1,1,0,0,3,0},
{0,0,2,4,0,0,0,1,0},
{0,2,0,0,8,0,4,3,10},
{0,0,3,0,0,4,0,0,0},
},
		// 11x11
{
{0,0,0,0,0,3,0,0,0,0,0},
{0,0,11,2,0,0,9,3,0,0,3},
{0,0,0,3,1,0,2,0,0,0,0},
{4,1,2,3,0,4,0,0,4,0,0},
{5,0,0,0,4,0,1,0,4,3,0},
{0,0,0,7,4,0,1,0,0,7,0},
{0,8,0,0,0,0,3,0,1,0,3},
{0,3,0,0,5,2,3,0,0,0,2},
{0,0,0,3,1,0,2,8,0,0,0},
{0,3,4,0,7,0,0,7,0,0,0},
{4,2,0,4,0,3,0,0,5,7,0},
},
{
{1,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0},
{0,0,10,10,0,0,0,4,5,0,0},
{0,4,1,0,8,0,0,0,0,0,5},
{8,0,0,0,0,0,6,0,0,0,0},
{0,0,0,0,13,0,0,0,2,0,3},
{0,0,0,0,4,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0},
{0,0,4,0,0,0,0,3,0,0,0},
{4,1,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0},
},
		// 13x13
{
{2,4,0,0,6,7,3,4,0,0,3,0,1},
{0,0,2,0,3,0,0,6,0,0,8,11,3},
{0,3,0,8,0,0,2,0,0,0,0,0,4},
{2,0,0,0,0,0,0,0,0,3,2,0,0},
{0,6,0,8,0,3,0,0,0,0,0,0,1},
{0,3,0,2,0,0,9,0,0,0,0,5,6},
{1,9,4,0,0,2,4,0,0,0,3,2,0},
{2,3,0,4,0,0,0,6,2,0,1,0,3},
{0,0,0,0,0,6,0,0,0,2,2,0,8},
{7,2,4,2,0,0,6,4,1,0,0,0,7},
{0,0,0,11,0,0,0,0,3,4,0,9,0},
{0,0,0,0,1,4,3,4,0,0,0,3,11},
{0,0,4,7,7,0,0,2,0,2,5,0,1},
},
{
{0,0,1,4,0,0,9,0,0,0,12,0,1},
{0,0,0,0,0,0,0,0,0,1,0,0,0},
{1,0,0,0,0,0,2,0,0,2,0,0,0},
{0,0,0,0,0,9,4,0,0,0,6,0,0},
{0,6,9,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,1,6,10,0,1,4},
{0,3,0,0,0,1,0,0,0,0,0,2,0},
{0,0,0,1,3,0,0,0,0,9,0,0,0},
{9,0,0,3,3,0,0,0,0,3,4,0,0},
{0,1,4,0,0,0,0,0,0,5,0,1,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0},
{2,0,0,0,0,3,3,0,0,0,0,0,10},
{0,0,0,0,0,0,0,0,0,4,0,0,0},
},
		// 15x13
{
	{0, 0, 0, 4, 0, 0, 0, 6, 0, 0, 0, 0, 2, 2, 0},
	{ 0,2,12,0,3,0,0,0,0,26,0,0,0,0,4 },
	{ 0,0,0,0,0,0,0,0,0,0,0,0,2,2,0 },
	{ 0,0,0,0,0,0,0,3,3,1,0,0,0,0,0 },
	{ 0,0,0,0,0,1,3,0,0,6,4,0,0,0,0 },
	{ 0,0,0,0,0,0,0,5,4,0,0,3,0,0,0 },
	{ 9,12,0,0,0,4,1,6,0,0,0,0,0,0,0 },
	{ 0,0,0,0,0,0,0,0,4,0,0,0,0,0,0 },
	{ 0,3,0,0,0,2,0,0,0,7,0,4,0,0,0 },
	{ 0,0,2,0,0,9,2,0,0,0,0,0,0,0,0 },
	{ 0,2,0,0,2,16,0,8,0,2,0,0,0,0,7 },
	{ 0,0,5,0,6,0,0,0,0,0,8,0,0,0,0 },
	{ 0,4,0,0,0,0,0,0,1,2,3,0,0,0,0 },
},
{
{0,0,0,0,0,0,0,10,3,0,0,0,0,2,0},
{0,0,0,0,0,0,0,0,0,0,0,1,0,0,0},
{0,1,0,3,0,0,0,0,4,0,0,0,4,0,0},
{0,0,0,10,3,8,11,0,0,0,0,0,2,0,0},
{0,4,0,0,0,0,0,0,0,0,0,2,0,0,1},
{0,4,2,0,0,0,4,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,5,0,10,0,0,1,0,0},
{0,0,0,0,0,0,0,0,3,0,0,2,8,0,15},
{0,1,0,0,0,0,0,0,0,0,0,0,11,0,0},
{0,0,0,0,0,3,0,0,0,0,0,1,0,2,0},
{0,0,0,11,0,0,0,0,0,0,0,0,0,0,2},
{8,0,0,0,0,4,0,0,0,0,0,4,2,0,4},
{0,0,0,0,0,0,0,0,0,0,0,0,0,8,1},
},

		// 17x13
{
{0,0,0,3,0,0,0,0,5,0,0,0,0,0,0,0,0},
{0,0,0,0,3,0,0,0,4,2,0,3,0,0,0,0,0},
{6,0,3,0,0,0,6,0,30,0,1,8,6,10,0,0,0},
{0,7,0,1,4,0,0,1,0,0,0,0,0,0,0,4,4},
{0,2,0,0,1,0,0,0,0,0,0,0,0,0,0,0,3},
{0,8,0,0,0,0,0,3,0,0,36,0,1,0,0,2,0},
{6,0,0,0,8,2,8,0,0,2,0,0,0,0,0,0,0},
{8,1,0,0,0,0,4,1,0,0,0,0,0,6,7,0,0},
{3,5,0,0,0,0,0,0,0,0,5,0,0,4,0,0,1},
{3,0,0,2,0,4,0,0,0,0,9,0,0,0,8,16,24},
{0,1,0,0,1,1,0,0,2,0,0,0,0,0,6,1,0},
{0,3,4,0,3,4,0,10,0,0,0,0,5,5,8,4,4},
{8,0,0,0,0,0,17,0,0,10,0,2,0,0,2,0,0},
},
{
{0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,1,4,0,0,0},
{0,6,0,0,0,0,8,0,10,0,0,0,0,2,2,3,0},
{0,0,0,0,0,4,0,8,3,0,0,0,0,0,0,0,0},
{0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0},
{4,0,8,1,0,0,7,0,0,0,0,0,5,3,0,0,0},
{0,0,3,0,1,0,0,3,0,0,3,0,3,0,8,0,0},
{0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,1,0},
{0,6,0,0,0,0,0,1,0,2,0,1,0,0,0,0,0},
{0,0,2,0,1,3,0,1,0,4,0,0,6,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0},
{4,8,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0},
{4,0,0,0,0,0,0,0,0,0,0,0,3,0,0,6,3},
},

		// 19x13
{
{0,0,0,0,4,0,0,4,0,0,8,0,6,0,0,0,0,0,4},
{0,0,0,0,0,2,0,6,0,0,0,0,0,0,0,0,0,3,1},
{2,0,8,3,0,0,0,5,0,4,0,0,0,0,0,2,1,4,0},
{0,0,1,0,4,0,0,0,0,0,0,1,0,0,0,18,10,0,0},
{0,0,1,0,0,0,0,3,0,2,0,0,0,7,4,0,0,4,3},
{0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,3,2,0,2},
{0,0,0,1,0,1,0,0,0,2,2,0,0,4,0,0,10,1,0},
{3,0,0,0,0,0,0,4,1,0,0,0,0,0,4,0,0,1,0},
{2,0,2,0,0,0,0,1,0,0,4,1,0,3,0,0,0,3,3},
{0,0,0,0,4,0,1,1,3,0,0,0,0,0,0,0,0,0,0},
{0,0,0,4,0,0,4,2,4,0,0,0,0,0,0,4,0,0,0},
{0,0,2,0,3,22,0,0,0,0,0,2,7,0,0,0,0,0,1},
{0,9,0,0,6,0,0,0,0,0,0,0,0,0,5,1,4,0,8},
},
{
{0,0,0,0,0,0,0,3,0,0,0,2,0,4,0,0,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0},
{5,0,0,3,0,0,0,0,0,0,0,2,1,0,0,0,0,0,0},
{0,0,4,0,0,0,0,0,14,0,0,0,0,2,0,7,0,0,0},
{0,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,2},
{0,0,3,3,0,0,0,0,0,3,0,0,0,0,0,0,2,0,0},
{5,0,0,0,0,0,6,0,2,2,0,0,0,0,0,0,0,0,0},
{0,0,2,0,0,0,0,0,4,0,0,5,0,0,0,0,0,4,1},
{3,0,4,0,0,0,0,0,0,0,0,7,2,0,0,0,1,0,3},
{0,0,1,0,0,4,11,0,3,0,0,0,0,11,3,0,0,0,0},
{1,0,2,8,0,0,0,0,0,0,4,0,0,3,1,0,0,0,0},
{3,0,11,0,0,0,0,0,0,0,0,0,0,3,0,7,0,0,0},
{0,0,0,0,0,2,0,0,0,4,0,0,0,0,0,0,0,1,0},
},

		// 19x15
{
{0,0,0,0,0,0,0,0,16,1,0,5,0,3,0,0,0,4,0},
{0,2,3,0,0,5,0,0,0,0,0,0,0,6,0,0,0,1,0},
{0,9,1,0,0,0,0,4,2,2,0,1,8,2,0,4,24,10,13},
{0,0,3,0,0,0,0,2,0,0,4,0,11,0,0,0,2,1,1},
{0,4,0,1,0,0,0,0,0,0,0,0,0,0,0,4,0,0,2},
{0,3,12,0,4,0,0,0,0,0,0,0,10,0,0,0,0,0,0},
{0,0,2,0,0,0,9,0,0,0,0,0,8,4,0,0,0,0,0},
{3,0,0,0,0,2,0,0,6,0,3,0,6,0,0,0,0,0,0},
{1,0,0,0,0,0,0,2,5,0,0,12,2,4,0,0,7,0,1},
{6,4,4,0,0,8,0,3,2,0,0,0,9,0,0,0,0,0,0},
{0,0,0,0,0,0,4,8,0,0,2,0,0,8,0,0,0,0,2},
{0,4,18,0,0,0,0,0,0,0,4,1,2,0,0,0,0,8,3},
{2,0,7,0,7,0,0,9,0,0,0,0,0,0,0,2,0,0,0},
{0,0,0,0,7,2,0,0,1,0,0,0,0,0,0,11,0,30,0},
{1,0,0,0,0,0,0,7,0,0,0,3,0,0,0,0,0,0,0},
},
{
{0,0,0,0,14,0,0,0,0,0,0,0,3,7,0,0,0,0,0},
{0,2,5,7,2,0,0,0,6,0,0,0,1,0,0,3,0,0,1},
{0,7,0,2,0,0,0,0,0,0,0,10,0,0,4,2,0,0,0},
{0,0,0,4,0,0,0,5,0,0,0,0,0,0,0,0,0,2,0},
{0,0,5,0,4,0,0,3,4,0,0,0,3,0,0,0,0,7,0},
{0,0,0,0,0,0,3,0,6,0,0,5,0,4,0,0,0,0,0},
{2,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,3,0},
{0,0,7,0,4,0,0,0,0,0,1,0,0,0,8,0,0,0,0},
{4,0,0,0,0,0,0,0,7,0,7,0,0,0,0,0,8,0,3},
{8,0,0,0,0,0,2,6,2,0,0,0,0,0,3,0,0,0,0},
{0,0,0,0,0,0,0,12,0,0,0,4,0,0,8,0,0,0,0},
{0,0,4,0,0,0,0,0,0,13,0,2,0,0,0,0,0,1,0},
{0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,3,0,2,0},
{0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,7,0},
{0,0,0,0,0,0,4,0,0,0,2,0,0,0,0,0,2,0,3},
},

		// 19x17
{
{0,2,0,0,0,4,4,0,0,4,0,0,1,6,0,1,4,0,0},
{1,0,0,2,0,0,0,0,0,0,0,0,9,3,0,0,0,0,0},
{3,0,0,4,0,9,1,0,0,1,0,0,0,6,0,0,0,0,0},
{0,4,0,0,4,4,0,0,0,0,0,0,12,0,0,0,0,1,0},
{0,0,0,3,0,6,0,0,0,0,3,0,0,11,17,0,0,0,0},
{6,0,1,0,0,6,0,0,1,0,0,0,0,0,0,0,0,0,0},
{0,1,0,1,0,0,0,0,0,0,6,0,0,1,0,0,0,2,0},
{1,0,10,0,0,2,2,0,3,4,8,0,0,9,11,1,0,16,0},
{3,0,0,0,0,0,0,0,4,0,0,7,0,0,7,0,0,0,0},
{0,6,0,1,0,0,0,0,3,5,0,0,2,4,0,0,0,0,0},
{0,0,0,0,0,6,0,3,6,0,10,6,0,0,0,0,0,0,2},
{3,0,0,4,4,0,2,0,0,0,1,0,0,1,2,16,11,0,0},
{7,0,0,3,0,0,0,0,0,10,12,0,0,0,0,0,0,0,0},
{0,2,0,0,0,0,2,2,0,0,0,0,0,0,0,4,1,0,1},
{0,0,0,0,0,0,0,1,2,6,3,0,0,0,0,0,7,0,0},
{0,0,1,0,4,8,0,0,0,0,0,6,0,0,0,6,0,0,0},
{0,0,0,0,0,0,0,2,0,2,0,0,0,7,0,0,0,0,2},
},
{
{3,4,0,0,3,0,0,0,0,6,0,4,4,0,0,0,4,0,0},
{4,0,0,5,0,0,0,0,7,3,0,0,0,0,0,0,0,0,0},
{0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,1,0},
{0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0},
{0,1,4,0,0,0,0,0,0,0,0,2,0,0,0,0,9,0,0},
{0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,1,0,4,0},
{4,0,0,5,0,0,0,0,0,0,0,11,0,0,0,0,0,3,0},
{2,0,7,0,0,11,0,0,0,0,0,0,5,0,7,0,0,0,0},
{9,0,0,0,1,0,1,15,0,0,0,0,1,0,0,0,1,4,3},
{0,3,0,0,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0},
{0,0,2,4,0,0,0,0,0,0,0,0,4,0,1,0,0,0,0},
{0,2,0,0,0,7,0,4,0,0,0,0,0,0,0,0,0,9,0},
{0,0,6,0,0,0,2,0,1,0,0,0,0,0,0,0,0,1,0},
{3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,9,0,0,3},
{0,0,0,0,0,1,0,0,0,0,0,0,9,0,0,0,7,0,0},
{0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,1,0},
{0,0,0,0,0,0,0,0,6,3,0,0,0,0,0,0,1,0,0},
},
	};
	static TZona dimensoes[] = {
		{5,5},{5,5}, 
		{7,7},{7,7}, 
		{9,9},{9,9},
		{11,11},{11,11},
		{13,13},{13,13}, 
		{13,15},{13,15},
		{13,17},{13,17},
		{13,19},{13,19},
		{15,19},{15,19},
		{17,19},{17,19},
	};

	altura = dimensoes[id].linha;
	largura = dimensoes[id].coluna;
	familias = 0;
	for (int i = 0; i < altura; i++)
		for (int j = 0; j < largura; j++) 
			familias += (CEstacao::populacao[i][j] = populacao[id][i][j]);
}


